
Class Main
    Dim DigToggle As Integer = 0
    Private WithEvents Arduino As Arduino_Net.Arduino

    Private Sub Main_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        Arduino = New Arduino_Net.Arduino("Com8", "9600")

        '-------
        'the following commands are not necessary for a Duemilanove:
        Arduino.DigitalCount = 14
        Arduino.AnalogCount = 6
        Arduino.PWMPorts = New Integer() {3, 5, 6, 9, 10, 11}
        '-------

        AddHandler Arduino.DigitalDataReceived, AddressOf ArduinoDigitalData
        AddHandler Arduino.AnalogDataReceived, AddressOf ArduinoAnalogData
        AddHandler Arduino.LogMessageReceived, AddressOf WriteLog
        AddHandler Arduino.ConnectionLost, AddressOf ConnectionLost
        'AddHandler Arduino.WatchdogReceived, AddressOf Watchdog


        If Arduino.StartCommunication() = True Then
            'Enable digital pin 2 as input with trigger
            Arduino.SetDigitalDirection(2, Arduino_Net.Arduino.DigitalDirection.Input)
            Arduino.EnableDigitalPort(2, True)
            Arduino.EnableDigitalTrigger(2, True)

            'Enable digital pin 3 as input with trigger
            Arduino.SetDigitalDirection(3, Arduino_Net.Arduino.DigitalDirection.Input)
            Arduino.EnableDigitalPort(3, True)
            Arduino.EnableDigitalTrigger(3, True)

            'enable analog pin 0 with threshold
            'Arduino.EnableAnalogPort(0, True)
            'Arduino.EnableAnalogTrigger(0, 20)

            'enable digital pin 13 as output
            Arduino.SetDigitalDirection(13, Arduino_Net.Arduino.DigitalDirection.DigitalOutput)
            Arduino.EnableDigitalPort(13, True)

            'enable digital pin 5 as PWM
            Arduino.SetDigitalDirection(5, Arduino_Net.Arduino.DigitalDirection.PWMOutput)
            Arduino.EnableDigitalPort(5, True)

            'enable analog pin 1 without threshold
            Arduino.EnableAnalogPort(1, True)
            Arduino.EnableAnalogTrigger(1, 0)

            'enable servo pin 9
            Arduino.EnableServoPort(9, True)

            'enable servo pin 10
            Arduino.EnableServoPort(10, True)
        End If
    End Sub

    Delegate Sub WriteLogCallback(ByVal LogText As String)

    Private Sub WriteLog(ByVal LogText As String)
        If TextBox1.InvokeRequired Then
            Dim d As New WriteLogCallback(AddressOf WriteLog)
            Me.Invoke(d, New Object() {LogText})
        Else
            TextBox1.Text += LogText + vbCrLf
            TextBox1.SelectionStart = TextBox1.TextLength
            TextBox1.ScrollToCaret()
        End If
    End Sub

    Delegate Sub ArduinoDigitalDataCallback(ByVal PortNr As Integer, ByVal Value As Integer)

    Private Sub ArduinoDigitalData(ByVal PortNr As Integer, ByVal Value As Integer)
        If Label1.InvokeRequired Then
            Dim d As New ArduinoDigitalDataCallback(AddressOf ArduinoDigitalData)
            Me.Invoke(d, New Object() {PortNr, Value})
        Else
            WriteLog("Digital data:  port: " + PortNr.ToString + " Value: " + Value.ToString)
            Select Case PortNr
                Case 2
                    If Value = 0 Then
                        Label1.BackColor = Drawing.Color.Black
                    Else
                        Label1.BackColor = Drawing.Color.Red
                    End If
                Case 3
                    If Value = 0 Then
                        Label2.BackColor = Drawing.Color.Black
                    Else
                        Label2.BackColor = Drawing.Color.Red
                    End If
            End Select
        End If
    End Sub

    Delegate Sub ArduinoAnalogDataCallback(ByVal PortNr As Integer, ByVal Value As Integer)

    Private Sub ArduinoAnalogData(ByVal PortNr As Integer, ByVal Value As Integer)
        If Label1.InvokeRequired Then
            Dim d As New ArduinoAnalogDataCallback(AddressOf ArduinoAnalogData)
            Me.Invoke(d, New Object() {PortNr, Value})
        Else
            WriteLog("Analog data:  port: " + PortNr.ToString + " Value: " + Value.ToString)
            Select Case PortNr
                Case 0
                    Label3.Text = Value.ToString
            End Select
        End If

        'process analog data produced by analogtimer1
        If PortNr = 1 Then
            'convert value to distance...needs testing

            'http://www.acroname.com/robotics/info/articles/irlinear/irlinear.html

            Dim Distance As Integer 'distance in cm
            If Value > 50 Then
                Distance = (6787 / (Value - 3)) - 4
                If Distance > 80 Then Distance = 80
            Else
                Distance = 0
            End If

            AnalogValue1.Text = Distance.ToString + " cm"
            AnalogSlider1.Value = Distance
            
        End If
    End Sub

    Private Sub Watchdog()
        WriteLog("Watchdog")
    End Sub

    Private Sub ConnectionLost()
        WriteLog("Connection is lost")
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)

    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        'get value of digital pin 2
        Arduino.GetDigitalValue(2)
    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        'get value of analog pin 0
        Arduino.GetAnalogValue(0)
    End Sub

    Private Sub Button1_Click_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If DigToggle = 0 Then
            DigToggle = 1
            Arduino.SetDigitalValue(13, 1)
        Else
            DigToggle = 0
            Arduino.SetDigitalValue(13, 0)
        End If
    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        'Dim PWMRnd As Integer
        'PWMRnd = Rnd() * 255
        'Arduino.SetDigitalValue(5, PWMRnd)

        For i As Integer = 0 To 255
            Arduino.SetDigitalValue(5, i)
        Next
        For i As Integer = 255 To 0 Step -1
            Arduino.SetDigitalValue(5, i)
        Next

    End Sub

    Private Sub Button5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click
        If AnalogTimer1.Enabled = False Then
            Button5.Text = "PollAnalogValue(1) *"
            AnalogTimer1.Enabled = True
        Else
            Button5.Text = "PollAnalogValue(1)"
            AnalogTimer1.Enabled = False
        End If
    End Sub

    Private Sub AnalogTimer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AnalogTimer1.Tick
        Arduino.GetAnalogValue(1)
    End Sub

    Private Sub TrackBar1_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ServoTrackBar1.Scroll
        ServoVal1.Text = ServoTrackBar1.Value.ToString
        Arduino.SetServoValue(10, ServoTrackBar1.Value)
    End Sub

    Private Sub ServoTrackBar_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ServoTrackBar2.Scroll
        ServoVal2.Text = ServoTrackBar2.Value.ToString
        Arduino.SetServoValue(9, ServoTrackBar2.Value)
    End Sub

End Class